/**
 * FLL - Level 3
 *
 * Project: IKI Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the eki_read functionality.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_read_eki_eki_read_h
#define _iki_read_eki_eki_read_h

// Libc includes.
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

// FLL-0 includes.
#include <fll/level_0/type.h>
#include <fll/level_0/status.h>
#include <fll/level_0/memory.h>
#include <fll/level_0/string.h>
#include <fll/level_0/utf.h>
#include <fll/level_0/color.h>
#include <fll/level_0/compare.h>
#include <fll/level_0/console.h>
#include <fll/level_0/conversion.h>
#include <fll/level_0/file.h>
#include <fll/level_0/iki.h>
#include <fll/level_0/pipe.h>
#include <fll/level_0/print.h>
#include <fll/level_0/rip.h>
#include <fll/level_0/signal.h>

#ifndef _di_thread_support_
  #include <fll/level_0/thread.h>
#endif // _di_thread_support_

// FLL-1 includes.
#include <fll/level_1/conversion.h>
#include <fll/level_1/iki.h>
#include <fll/level_1/print.h>

// FLL-2 includes.
#include <fll/level_2/error.h>
#include <fll/level_2/file.h>
#include <fll/level_2/print.h>
#include <fll/level_2/program.h>

// IKI Read includes.
#include <program/iki_read/main/common/define.h>
#include <program/iki_read/main/common/enumeration.h>
#include <program/iki_read/main/common/print.h>
#include <program/iki_read/main/common/string.h>
#include <program/iki_read/main/common/type.h>
#include <program/iki_read/main/common.h>
#include <program/iki_read/main/identify.h>
#include <program/iki_read/main/print/data.h>
#include <program/iki_read/main/print/error.h>
#include <program/iki_read/main/print/message.h>
#include <program/iki_read/main/process.h>
#include <program/iki_read/main/signal.h>
#include <program/iki_read/main/thread.h>
#include <program/iki_read/eki/common.h>
#include <program/iki_read/eki/identify.h>
#include <program/iki_read/eki/print.h>
#include <program/iki_read/eki/process.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Delete the program main setting data.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This does not alter main.setting.state.status.
 *
 * @see f_iki_data_delete()
 * @see f_memory_array_resize()
 * @see f_memory_arrays_resize()
 */
#ifndef _di_iki_read_eki_delete_data_
  extern void iki_read_eki_delete_data(iki_read_main_t * const main);
#endif // _di_iki_read_eki_delete_data_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_read_eki_eki_read_h
