#include "eki_read.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_read_eki_print_data_
  void iki_read_eki_print_data(fl_print_t * const print, const f_number_unsigned_t index) {

    if (!print || !print->custom) return;

    iki_read_main_t * const main = (iki_read_main_t *) print->custom;

    if (!main->data) return;

    f_iki_eki_t * const data = (f_iki_eki_t *) main->data;

    if (index >= data->vocabularys.used) return;

    // @todo This needs to handle order of operations such that it does not return immediately after say a reassign or a substitution.
    //       When a substitution or a re-assign is matched, then all things to the right can be ignored.

    f_number_unsigned_t i = 0;
    f_number_unsigned_t j = main->setting.reassign.used;
    f_range_t range = f_range_t_initialize;

    for (; i < data->vocabularys.array[index].used; ++i) {

      if (main->setting.reassign.used && (main->setting.flag & (iki_read_main_flag_content_d | iki_read_main_flag_literal_d))) {
        j = iki_read_identify_substitution(main, data->vocabularys.array[index].array[i], data->content.array[index], &main->setting.reassign);
        if (j < main->setting.reassign.used) break;
      }
    } // for

    if (j < main->setting.reassign.used) {
      if (main->setting.flag & iki_read_main_flag_content_d) {
        f_print_dynamic(main->setting.reassign.array[j].c, main->program.output.to);
      }
      else {
        range.start = data->variable.array[index].start;
        range.stop = data->content.array[index].start - 1;

        f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);

        f_print_dynamic(main->setting.reassign.array[j].c, main->program.output.to);

        range.start = data->content.array[index].stop + 1;
        range.stop = data->variable.array[index].stop;

        f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);
      }

      return;
    }

    j = main->setting.substitute.used;

    for (i = 0; i < data->vocabularys.array[index].used; ++i) {

      if (main->setting.substitute.used && (main->setting.flag & (iki_read_main_flag_content_d | iki_read_main_flag_literal_d))) {
        j = iki_read_identify_substitution(main, data->vocabularys.array[index].array[i], data->content.array[index], &main->setting.substitute);
        if (j < main->setting.substitute.used) break;
      }
    } // for

    if (j < main->setting.substitute.used) {
      if (main->setting.flag & iki_read_main_flag_content_d) {
        iki_read_print_data_wrap_prepend(&main->program.output, index);

        f_print_dynamic(main->setting.substitute.array[j].c, main->program.output.to);

        iki_read_print_data_wrap_append(&main->program.output, index);
      }
      else {
        range.start = data->variable.array[index].start;
        range.stop = data->content.array[index].start - 1;

        f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);

        iki_read_print_data_wrap_prepend(&main->program.output, index);

        f_print_dynamic(main->setting.substitute.array[j].c, main->program.output.to);

        iki_read_print_data_wrap_append(&main->program.output, index);

        range.start = data->content.array[index].stop + 1;
        range.stop = data->variable.array[index].stop;

        f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);
      }
    }
    else if (main->setting.replace.used && main->setting.map_replacess.array[index].used) {
      if (main->setting.flag & iki_read_main_flag_content_d) {
        iki_read_print_data_wrap_prepend(&main->program.output, index);

        for (j = 0; j < main->setting.map_replacess.array[index].used; ++j) {

          if (main->setting.map_replacess.array[index].array[j] < main->setting.replace.used) {
            f_print_dynamic(main->setting.replace.array[main->setting.map_replacess.array[index].array[j]].value, main->program.output.to);
          }
        } // for

        iki_read_print_data_wrap_append(&main->program.output, index);
      }
      else if (main->setting.flag & iki_read_main_flag_object_d) {
        if (main->setting.flag & iki_read_main_flag_select_d) {
          if (main->setting.select < main->setting.map_replacess.array[index].used) {
            f_print_dynamic(main->setting.replace.array[main->setting.map_replacess.array[index].array[main->setting.select]].key, main->program.output.to);
          }
        }
        else {
          for (j = 0; j < main->setting.map_replacess.array[index].used; ++j) {

            if (main->setting.map_replacess.array[index].array[j] < main->setting.replace.used) {
              f_print_dynamic(main->setting.replace.array[main->setting.map_replacess.array[index].array[j]].key, main->program.output.to);
            }
          } // for
        }
      }
      else {
        range.start = data->variable.array[index].start;
        range.stop = data->content.array[index].start - 1;

        f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);

        iki_read_print_data_wrap_prepend(&main->program.output, index);

        for (j = 0; j < main->setting.map_replacess.array[index].used; ++j) {

          if (main->setting.map_replacess.array[index].array[j] < main->setting.replace.used) {
            f_print_dynamic(main->setting.replace.array[main->setting.map_replacess.array[index].array[j]].value, main->program.output.to);
          }
        } // for

        iki_read_print_data_wrap_append(&main->program.output, index);

        range.start = data->content.array[index].stop + 1;
        range.stop = data->variable.array[index].stop;

        f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);
      }
    }
    else if (main->setting.flag & iki_read_main_flag_content_d) {
      iki_read_print_data_wrap_prepend(&main->program.output, index);

      f_print_dynamic_partial(main->cache.buffer, data->content.array[index], main->program.output.to);

      iki_read_print_data_wrap_append(&main->program.output, index);
    }
    else if (main->setting.flag & iki_read_main_flag_object_d) {
      if (main->setting.flag & iki_read_main_flag_select_d) {
        if (main->setting.select < data->vocabularys.array[index].used) {
          f_print_dynamic_partial(main->cache.buffer, data->vocabularys.array[index].array[main->setting.select], main->program.output.to);
        }
      }
      else {
        for (j = 0; j < data->vocabularys.array[index].used; ) {

          f_print_dynamic_partial(main->cache.buffer, data->vocabularys.array[index].array[j], main->program.output.to);

          if (++j < data->vocabularys.array[index].used) {
            f_print_dynamic_raw(f_string_space_s, print->to);
          }
        } // for
      }
    }
    else {
      range.start = data->variable.array[index].start;
      range.stop = data->content.array[index].start - 1;

      f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);

      iki_read_print_data_wrap_prepend(&main->program.output, index);

      f_print_dynamic_partial(main->cache.buffer, data->content.array[index], main->program.output.to);

      iki_read_print_data_wrap_append(&main->program.output, index);

      range.start = data->content.array[index].stop + 1;
      range.stop = data->variable.array[index].stop;

      f_print_dynamic_partial(main->cache.buffer, range, main->program.output.to);
    }
  }
#endif // _di_iki_read_eki_print_data_

#ifndef _di_iki_read_eki_print_message_help_
  void iki_read_eki_print_message_help(fl_print_t * const print) {

    if (!print) return;

    f_file_stream_lock(print->to);

    fll_program_print_help_header(print, iki_read_program_name_long_s, iki_read_program_version_s);

    iki_read_print_message_help(print);

    f_print_dynamic_raw(f_string_eol_s, print->to);

    fll_program_print_help_usage(print, iki_read_program_name_s, f_string_empty_s);

    iki_read_print_message_help_note(print);

    f_file_stream_flush(print->to);
    f_file_stream_unlock(print->to);
  }
#endif // _di_iki_read_eki_print_message_help_

#ifdef __cplusplus
} // extern "C"
#endif
