#include "iki_read.h"
#include "../main/iki_read.h"

int main(const int argc, const f_string_t *argv, const f_string_t *envp) {

  iki_read_main_t data = iki_read_main_t_initialize;
  f_iki_data_t data_iki = f_iki_data_t_initialize;

  data.data = (void *) &data_iki;

  data.program.debug.flag |= iki_read_print_flag_debug_d | iki_read_print_flag_out_d;
  data.program.error.flag |= iki_read_print_flag_error_d | iki_read_print_flag_out_d;
  data.program.output.flag |= iki_read_print_flag_out_d;
  data.program.message.flag |= iki_read_print_flag_message_d | iki_read_print_flag_out_d;
  data.program.warning.flag |= iki_read_print_flag_warning_d | iki_read_print_flag_out_d;
  data.program.error.custom = (void *) &data;
  data.program.debug.custom = (void *) &data;
  data.program.message.custom = (void *) &data;
  data.program.output.custom = (void *) &data;
  data.program.warning.custom = (void *) &data;

  data.setting.state.custom = (void *) &data;
  data.setting.state.handle = &fll_program_standard_signal_handle;

  data.callback.identify_alteration = &iki_read_iki_identify_alteration;
  data.callback.print_data = &iki_read_iki_print_data;
  data.callback.print_help = &iki_read_iki_print_message_help;
  data.callback.process_buffer_ranges = &iki_read_iki_process_buffer_ranges;
  data.callback.process_buffer_ranges_whole = &iki_read_iki_process_buffer_ranges_whole;
  data.callback.process_buffer_total = &iki_read_iki_process_buffer_total;
  data.callback.delete_data = &iki_read_iki_delete_data;

  f_console_parameter_t parameters[] = iki_read_console_parameter_t_initialize;
  data.program.parameters.array = parameters;
  data.program.parameters.used = iki_read_parameter_total_d;
  data.program.environment = envp;

  if (f_pipe_input_exists()) {
    data.program.pipe = fll_program_data_pipe_input_e;
  }

  fll_program_standard_set_up(&data.program);

  f_file_umask_get(&data.program.umask);

  #ifdef _di_thread_support_
    {
      const f_console_arguments_t arguments = macro_f_console_arguments_t_initialize_1(argc, argv, envp);

      iki_read_setting_load(arguments, &data, &iki_read_iki_setting_load);
    }

    iki_read_main(&data);
  #else
    {
      f_thread_id_t id_signal;

      memset(&id_signal, 0, sizeof(f_thread_id_t));

      data.setting.state.status = f_thread_create(0, &id_signal, &iki_read_thread_signal, (void *) &data);

      if (F_status_is_error(data.setting.state.status)) {
        iki_read_print_error(&data.program.error, macro_iki_read_f(f_thread_create));
      }
      else {
        {
          const f_console_arguments_t arguments = macro_f_console_arguments_t_initialize_1(argc, argv, envp);

          iki_read_setting_load(arguments, &data, &iki_read_iki_setting_load);
        }

        if (!iki_read_signal_check(&data)) {
          iki_read_main(&data);
        }

        f_thread_cancel(id_signal);
        f_thread_join(id_signal, 0);
      }
    }
  #endif // _di_thread_support_

  iki_read_main_delete(&data);

  fll_program_standard_set_down(&data.program);

  return F_status_is_error(data.setting.state.status) ? 1 : 0;
}
