/**
 * FLL - Level 3
 *
 * Project: IKI Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the print functionality.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_read_iki_print_h
#define _iki_read_iki_print_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Print the given range at the given index for IKI read.
 *
 * This expects the caller to have the output locked appropriately.
 *
 * This detects and prints any applicable substitution matching the vocabulary at the given index.
 * If there is no substitution, then this prints the given range at the given index.
 *
 * @param print
 *   The output structure to print to.
 *
 *   The setting.map_wrapss is expected to be defined as a valid pointer to an array.
 *   The setting.map_replacess is expected to be defined as a valid pointer to an array.
 *
 *   This does not alter print.custom.setting.state.status.
 * @param index
 *   The index used to identify the desired range in variable, content, and ranges.
 *
 * @see f_print_dynamic()
 * @see f_print_dynamic_partial()
 *
 * @see iki_read_identify_substitution()
 * @see iki_read_print_data_wrap_append()
 * @see iki_read_print_data_wrap_prepend()
 */
#ifndef _di_iki_read_iki_print_data_
  extern void iki_read_iki_print_data(fl_print_t * const print, const f_number_unsigned_t index);
#endif // _di_iki_read_iki_print_data_

/**
 * Print help for IKI read.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 */
#ifndef _di_iki_read_iki_print_message_help_
  extern void iki_read_iki_print_message_help(fl_print_t * const print);
#endif // _di_iki_read_iki_print_message_help_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_read_iki_print_h
