/**
 * FLL - Level 3
 *
 * Project: IKI Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the process functionality.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_read_iki_process_h
#define _iki_read_iki_process_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Process the given range at the given index for IKI read.
 *
 * This expands all of the appropriate Variables so that post-expansion calculations may be performed.
 * This is particularly useful for preparing to print a single Line in Content and Literal modes.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   The setting.map_wrapss is expected to be defined as a valid pointer to an array.
 *   The setting.map_replacess is expected to be defined as a valid pointer to an array.
 *
 *   This alters setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but nothing done.
 *
 *     F_interrupt (with error bit) on (exit) signal received.
 *
 *     Errors (with error bit) from: f_string_dynamic_append_nulless().
 *     Errors (with error bit) from: f_string_dynamic_partial_append_nulless().
 *
 *     Errors (with error bit) from: iki_read_process_wrap_append().
 *     Errors (with error bit) from: iki_read_process_wrap_prepend().
 * @param index
 *   The index used to identify the desired range in variable, content, and ranges.
 *
 * @see f_string_dynamic_append_nulless()
 * @see f_string_dynamic_partial_append_nulless()
 *
 * @see iki_read_identify_substitution()
 * @see iki_read_process_wrap_append()
 * @see iki_read_process_wrap_prepend()
 */
#ifndef _di_iki_read_iki_process_buffer_at_
  extern void iki_read_iki_process_buffer_at(iki_read_main_t * const main, const f_number_unsigned_t index);
#endif // _di_iki_read_iki_process_buffer_at_

/**
 * Process a given buffer, printing the given range for IKI read.
 *
 * This prints error messages.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but nothing done.
 *
 *     F_parameter (with error bit) if a parameter is invalid.
 *     F_interrupt (with error bit) on (exit) signal received.
 *
 *     Errors (with error bit) from: iki_read_iki_process_buffer_ranges_prepare().
 * @param range_buffer
 *   The range within the buffer to process.
 *
 *   Must not be NULL.
 *
 * @see iki_read_iki_process_buffer_ranges_prepare()
 */
#ifndef _di_iki_read_iki_process_buffer_ranges_
  extern void iki_read_iki_process_buffer_ranges(iki_read_main_t * const main, f_range_t * const range_buffer);
#endif // _di_iki_read_iki_process_buffer_ranges_

/**
 * Helper function for preparing the processing of the buffer ranges.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     Not directly changed on success.
 *
 *     Errors (with error bit) from: f_memory_arrays_resize().
 *     Errors (with error bit) from: fl_iki_read().
 * @param range_buffer
 *   The range within the buffer to process.
 *
 *   Must not be NULL.
 * @param data
 *   The IKI data structure.
 *
 *   Must not be NULL.
 *
 * @see f_memory_arrays_resize()
 * @see fl_iki_read()
 */
#ifndef _di_iki_read_iki_process_buffer_ranges_prepare_
  extern void iki_read_iki_process_buffer_ranges_prepare(iki_read_main_t * const main, f_range_t * const range_buffer, f_iki_data_t * const data);
#endif // _di_iki_read_iki_process_buffer_ranges_prepare_

/**
 * Process a given buffer, printing the given buffer in whole mode based on the given ranges for IKI read.
 *
 * The entire variable is replaced with the value from the associated ranges.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but nothing done.
 *
 *     F_interrupt (with error bit) on (exit) signal received.
 *
 *     Errors (with error bit) from: f_memory_arrays_resize().
 * @param range_buffer
 *   The range within the buffer to process.
 *
 *   Must not be NULL.
 *
 * @see f_memory_arrays_resize()
 */
#ifndef _di_iki_read_iki_process_buffer_ranges_whole_
  extern void iki_read_iki_process_buffer_ranges_whole(iki_read_main_t * const main, f_range_t * const range_buffer);
#endif // _di_iki_read_iki_process_buffer_ranges_whole_

/**
 * Process a given buffer, printing the total for IKI read.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 * @return
 *   F_okay on success.
 *
 *   Status codes (with error bit) are returned on any problem.
 */
#ifndef _di_iki_read_iki_process_buffer_total_
  extern void iki_read_iki_process_buffer_total(iki_read_main_t * const main);
#endif // _di_iki_read_iki_process_buffer_total_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_read_iki_process_h
