/**
 * FLL - Level 3
 *
 * Project: IKI Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common define types.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_read_common_define_h
#define _iki_read_common_define_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The program allocation defines.
 *
 * iki_read_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   An allocation step used for buffers that are anticipated to have large buffers.
 *   - pipe:    A buffer size used for processing piped data.
 *   - small:   An allocation step used for buffers that are anticipated to have small buffers.
 */
#ifndef _di_iki_read_allocation_d_
  #define iki_read_allocation_console_d 4
  #define iki_read_allocation_large_d   256
  #define iki_read_allocation_pipe_d    16384
  #define iki_read_allocation_small_d   16
#endif // _di_iki_read_allocation_d_

/**
 * The program block defines.
 *
 * iki_read_block_*_d:
 *   - max:        The max block read size before checking for interrupt.
 *   - read_small: The block read size for small files.
 *   - read_large: The block read size for large files.
 */
#ifndef _di_iki_read_block_d_
  #define iki_read_block_max_d        16777216
  #define iki_read_block_read_small_d 8192
  #define iki_read_block_read_large_d 65536
#endif // _di_iki_read_block_d_

/**
 * Flags passed to the main function or program.
 *
 * iki_read_main_flag_*_d:
 *   - none:                   No flags set.
 *   - at:                     Selecting at a specific index.
 *   - content:                Print Contents.
 *   - copyright:              Print copyright.
 *   - help:                   Print help.
 *   - literal:                Print as literal data (printing entire variable).
 *   - line:                   Selecting at a specific line.
 *   - name:                   Selecting using a specific Vocabulary name.
 *   - object:                 Print Objects.
 *   - pipe:                   Use the input pipe.
 *   - select:                 Using select.
 *   - reassign:               Using re-assignments.
 *   - replace:                Using replacements.
 *   - substitute:             Using substitutions.
 *   - total:                  Enable printing of "total" count.
 *   - version:                Print version.
 *   - version_copyright_help: A helper flag representing version, copyright, and help flag bits being set.
 *   - whole:                  Print all data.
 *   - wrap:                   Using wrapping.
 */
#ifndef _di_iki_read_main_flag_d_
  #define iki_read_main_flag_none_d                   0x0
  #define iki_read_main_flag_at_d                     0x1
  #define iki_read_main_flag_content_d                0x2
  #define iki_read_main_flag_copyright_d              0x4
  #define iki_read_main_flag_help_d                   0x8
  #define iki_read_main_flag_literal_d                0x10
  #define iki_read_main_flag_line_d                   0x20
  #define iki_read_main_flag_name_d                   0x40
  #define iki_read_main_flag_object_d                 0x80
  #define iki_read_main_flag_pipe_d                   0x100
  #define iki_read_main_flag_reassign_d               0x200
  #define iki_read_main_flag_replace_d                0x400
  #define iki_read_main_flag_select_d                 0x800
  #define iki_read_main_flag_substitute_d             0x1000
  #define iki_read_main_flag_total_d                  0x2000
  #define iki_read_main_flag_version_d                0x4000
  #define iki_read_main_flag_version_copyright_help_d 0x400c
  #define iki_read_main_flag_whole_d                  0x8000
  #define iki_read_main_flag_wrap_d                   0x10000
#endif // _di_iki_read_main_flag_d_

/**
 * Flags for fine-tuned print control.
 *
 * iki_read_print_flag_*_d:
 *   - none:    No flags set.
 *   - debug:   Stream is for debug printing.
 *   - error:   Stream is for error printing.
 *   - in:      Stream is a source file.
 *   - message: Stream is for message printing.
 *   - out:     Stream is a destination file.
 *   - warning: Stream is for warning printing.
 */
#ifndef _di_iki_read_print_flag_d_
  #define iki_read_print_flag_none_d    0x0
  #define iki_read_print_flag_debug_d   0x1
  #define iki_read_print_flag_error_d   0x2
  #define iki_read_print_flag_file_d    0x4
  #define iki_read_print_flag_in_d      0x8
  #define iki_read_print_flag_out_d     0x10
  #define iki_read_print_flag_message_d 0x20
  #define iki_read_print_flag_warning_d 0x40
#endif // _di_iki_read_print_flag_d_

/**
 * The program signal defines.
 *
 * iki_read_signal_*_d:
 *   - check:          Number of iterations before performing signal check in non-threaded signal handling.
 *   - check_failsafe: When using threads, how many consecutive failures to check signal before aborting (as a recursion failsafe).
 *   - check_tiny:     The tiny check.
 *   - check_short:    The short signal check.
 */
#ifndef _di_iki_read_signal_d_
  #define iki_read_signal_check_d          500000
  #define iki_read_signal_check_failsafe_d 20000
  #define iki_read_signal_check_tiny_d     4
  #define iki_read_signal_check_short_d    16
#endif // _di_iki_read_signal_d_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_read_common_define_h
