#include "iki_read.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_read_main_
  void iki_read_main(iki_read_main_t * const main) {

    if (!main || F_status_is_error(main->setting.state.status)) return;

    main->setting.state.status = F_okay;

    if (main->setting.flag & iki_read_main_flag_version_copyright_help_d) {
      if (main->setting.flag & iki_read_main_flag_help_d) {
        if (main->callback.print_help) {
          main->callback.print_help(&main->program.message);
        }
      }
      else if (main->setting.flag & iki_read_main_flag_version_d) {
        fll_program_print_version(&main->program.message, iki_read_program_version_s);
      }
      else if (main->setting.flag & iki_read_main_flag_copyright_d) {
        fll_program_print_copyright(&main->program.message, fll_program_copyright_year_author_s);
      }

      return;
    }

    f_file_t file = f_file_t_initialize;

    if (main->setting.flag & iki_read_main_flag_pipe_d) {
      file.id = F_type_descriptor_input_d;

      main->cache.buffer.used = 0;

      main->setting.state.status = f_file_read(file, &main->cache.buffer);

      if (F_status_is_error(main->setting.state.status)) {
        iki_read_print_error_file(&main->program.error, macro_iki_read_f(f_file_read), f_string_ascii_minus_s, f_file_operation_process_s, fll_error_file_type_file_e);
      }
      else {
        iki_read_process_buffer(main);
      }
    }

    if (F_status_is_error_not(main->setting.state.status) && main->setting.files.used) {
      off_t size_block = 0;
      off_t size_file = 0;
      off_t size_read = 0;

      for (f_number_unsigned_t i = 0; i < main->setting.files.used; ++i) {

        if (iki_read_signal_check(main)) return;

        file.stream = 0;
        file.id = -1;

        main->setting.state.status = f_file_stream_open(main->setting.files.array[i], f_string_empty_s, &file);

        if (F_status_is_error(main->setting.state.status)) {
          iki_read_print_error_file(&main->program.error, macro_iki_read_f(f_file_stream_open), main->setting.files.array[i], f_file_operation_process_s, fll_error_file_type_file_e);

          break;
        }

        main->setting.state.status = f_file_descriptor(&file);

        if (F_status_is_error(main->setting.state.status)) {
          iki_read_print_error_file(&main->program.error, macro_iki_read_f(f_file_descriptor), main->setting.files.array[i], f_file_operation_process_s, fll_error_file_type_file_e);

          break;
        }

        size_file = 0;

        main->setting.state.status = f_file_size_by_id(file, &size_file);

        if (F_status_is_error(main->setting.state.status)) {
          iki_read_print_error_file(&main->program.error, macro_iki_read_f(f_file_size_by_id), main->setting.files.array[i], f_file_operation_analyze_s, fll_error_file_type_file_e);

          break;
        }

        // Skip past empty files.
        if (!size_file) {
          f_file_stream_flush(file);
          f_file_stream_close(&file);

          continue;
        }

        // Enforce a max block read size to allow for interrupts to be processed between blocks.
        if (size_file > iki_read_block_max_d) {
          file.size_read = iki_read_block_read_large_d;
          size_block = iki_read_block_max_d;
        }
        else {
          file.size_read = iki_read_block_read_small_d;
          size_block = size_file;
        }

        // Pre-allocate entire file buffer plus space for the terminating NULL.
        main->setting.state.status = f_memory_array_increase_by(size_file + 1, sizeof(f_char_t), (void **) &main->cache.buffer.string, &main->cache.buffer.used, &main->cache.buffer.size);

        if (F_status_is_error(main->setting.state.status)) {
          iki_read_print_error_file(&main->program.error, macro_iki_read_f(f_memory_array_increase_by), main->setting.files.array[i], f_file_operation_process_s, fll_error_file_type_file_e);

          break;
        }

        for (size_read = 0; size_read < size_file; size_read += size_block) {

          if (size_file > iki_read_block_max_d && iki_read_signal_check(main)) break;

          main->setting.state.status = f_file_stream_read_until(file, size_block, &main->cache.buffer);
          if (F_status_is_error(main->setting.state.status)) break;
        } // for

        if (F_status_is_error(main->setting.state.status)) {
          if (F_status_set_fine(main->setting.state.status) != F_interrupt) {
            iki_read_print_error_file(&main->program.error, macro_iki_read_f(f_file_stream_read_until), main->setting.files.array[i], f_file_operation_process_s, fll_error_file_type_file_e);
          }

          break;
        }

        f_file_stream_flush(file);
        f_file_stream_close(&file);

        iki_read_process_buffer(main);
        if (F_status_is_error(main->setting.state.status)) break;

        main->cache.buffer.used = 0;
      } // for

      if (F_status_is_error(main->setting.state.status)) {
        f_file_stream_flush(file);
        f_file_stream_close(&file);
      }
    }
  }
#endif // _di_iki_read_main_

#ifdef __cplusplus
} // extern "C"
#endif
