/**
 * FLL - Level 3
 *
 * Project: IKI Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the print data functionality.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_read_print_data_h
#define _iki_read_print_data_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Print the data from the expand cache at the line.
 *
 * This requires that the expand cache is properly populated.
 * The expand cache may be empty.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This alters print.custom.setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but total is 0 or line is out of range.
 *
 *     Error (with error bit set) from f_string_dynamic_seek_line().
 *     Error (with error bit set) from iki_read_signal_check().
 *
 *     Error (with error bit) if status already has an error before start of function.
 */
#ifndef _di_iki_read_print_data_line_
  extern void iki_read_print_data_line(fl_print_t * const print);
#endif // _di_iki_read_print_data_line_

/**
 * Print the number that represents the total.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This alters print.custom.setting.state.status:
 *     F_okay on success and data was processed.
 *     F_data_not on success, but there was no data to process (0 is printed).
 *
 *     Signal from iki_read_signal_check().
 *
 *     Error (with error bit set) from f_string_dynamic_seek_line()
 *
 * @see f_string_dynamic_seek_line()
 *
 * @see iki_read_signal_check()
 */
#ifndef _di_iki_read_print_data_total_
  extern void iki_read_print_data_total(fl_print_t * const print, const f_number_unsigned_t);
#endif // _di_iki_read_print_data_total_

/**
 * Count and print the total from the data in the expand cache at the line.
 *
 * This requires that the expand cache is properly populated.
 * The expand cache may be empty.
 *
 * This only checks if the flag iki_read_main_flag_line_d is set.
 * Otherwise, the total lines from the expand cache is counted as-is.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This alters print.custom.setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but total is 0 or line is out of range.
 *
 *     Error (with error bit set) from f_string_dynamic_seek_line().
 *     Error (with error bit set) from iki_read_signal_check().
 *
 *     Error (with error bit) if status already has an error before start of function.
 *
 * @see f_string_dynamic_seek_line()
 * @see iki_read_signal_check()
 */
#ifndef _di_iki_read_print_data_line_total_expand_
  extern void iki_read_print_data_total_expand(fl_print_t * const print);
#endif // _di_iki_read_print_data_line_total_expand_

/**
 * Print the append part of the wrap at the given index, if valid.
 *
 * This expects the caller to have the output locked appropriately.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 * @param index
 *   The index within the setting->map_wrapss array to print.
 */
#ifndef _di_iki_read_print_data_wrap_append_
  extern void iki_read_print_data_wrap_append(fl_print_t * const print, const f_number_unsigned_t index);
#endif // _di_iki_read_print_data_wrap_append_

/**
 * Print the prepend part of the wrap at the given index, if valid.
 *
 * This expects the caller to have the output locked appropriately.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 * @param index
 *   The index within the setting->map_wrapss array to print.
 */
#ifndef _di_iki_read_print_data_wrap_prepend_
  extern void iki_read_print_data_wrap_prepend(fl_print_t * const print, const f_number_unsigned_t index);
#endif // _di_iki_read_print_data_wrap_prepend_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_read_print_data_h
