#include "iki_read.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_read_process_buffer_
  void iki_read_process_buffer(iki_read_main_t * const main) {

    if (F_status_is_error(main->setting.state.status)) return;

    f_range_t buffer_range = macro_f_range_t_initialize_2(main->cache.buffer.used);

    if (main->setting.flag & iki_read_main_flag_whole_d) {
      iki_read_process_line(main, &buffer_range);

      if (main->setting.flag & iki_read_main_flag_total_d) {
        if (main->callback.process_buffer_total) {
          main->callback.process_buffer_total(main);

          if (F_status_is_error_not(main->setting.state.status)) {
            main->setting.state.status = F_okay;
          }
        }

        return;
      }

      if (main->setting.state.status == F_data_not || main->setting.state.status == F_true && buffer_range.start > main->cache.buffer.used) {
        main->setting.state.status = F_data_not;

        return;
      }

      if (main->callback.process_buffer_ranges_whole) {
        main->callback.process_buffer_ranges_whole(main, &buffer_range);
      }
    }
    else {
      if (main->callback.process_buffer_ranges) {
        main->callback.process_buffer_ranges(main, &buffer_range);
      }
    }

    if (F_status_is_error_not(main->setting.state.status)) {
      main->setting.state.status = F_okay;
    }
  }
#endif // _di_iki_read_process_buffer_

#ifndef _di_iki_read_process_buffer_at_newline_
  void iki_read_process_buffer_at_newline(iki_read_main_t * const main) {

    if (!main || main->setting.state.status != F_okay) return;

    main->setting.state.status = f_string_dynamic_append(f_string_eol_s, &main->cache.expand);
  }
#endif // _di_iki_read_process_buffer_at_newline_

#ifndef _di_iki_read_process_line_
  void iki_read_process_line(iki_read_main_t * const main, f_range_t * const range) {

    if (!(main->setting.flag & iki_read_main_flag_line_d)) {
      main->setting.state.status = F_false;

      return;
    }

    f_number_unsigned_t line = 0;

    range->start = 0;

    if (main->setting.line) {
      for (; line < main->setting.line && range->start < main->cache.buffer.used; ++range->start) {
        if (main->cache.buffer.string[range->start] == f_string_eol_s.string[0]) ++line;
      } // for
    }

    if (line == main->setting.line) {

      // Find where the found line ends and set that as the range stop.
      for (range->stop = range->start; range->stop < main->cache.buffer.used; ++range->stop) {
        if (main->cache.buffer.string[range->stop] == f_string_eol_s.string[0]) break;
      } // for

      main->setting.state.status = F_true;
    }
    else {
      main->setting.state.status = F_data_not;
    }
  }
#endif // _di_iki_read_process_line_

#ifndef _di_iki_read_process_wrap_append_
  void iki_read_process_wrap_append(iki_read_main_t * const main, const f_number_unsigned_t index) {

    if (!main || F_status_is_error(main->setting.state.status)) return;

    if (index >= main->setting.map_wrapss.used) {
      main->setting.state.status = F_data_not;

      return;
    }

    f_number_unsigned_t * const array = main->setting.map_wrapss.array[index].array;

    for (f_number_unsigned_t i = 0; i < main->setting.map_wrapss.array[index].used; ++i) {

      if (array[i] < main->setting.wrap.used && main->setting.wrap.array[array[i]].c.used) {
        main->setting.state.status = f_string_dynamic_append_nulless(main->setting.wrap.array[array[i]].c, &main->cache.expand);
        if (F_status_is_error(main->setting.state.status)) return;
      }
    } // for

    main->setting.state.status = F_okay;
  }
#endif // _di_iki_read_process_wrap_append_

#ifndef _di_iki_read_process_wrap_prepend_
  void iki_read_process_wrap_prepend(iki_read_main_t * const main, const f_number_unsigned_t index) {

    if (!main || F_status_is_error(main->setting.state.status)) return;

    if (index >= main->setting.map_wrapss.used) {
      main->setting.state.status = F_data_not;

      return;
    }

    f_number_unsigned_t * const array = main->setting.map_wrapss.array[index].array;

    for (f_number_unsigned_t i = 0; i < main->setting.map_wrapss.array[index].used; ++i) {

      if (array[i] < main->setting.wrap.used && main->setting.wrap.array[array[i]].b.used) {
        main->setting.state.status = f_string_dynamic_append_nulless(main->setting.wrap.array[array[i]].b, &main->cache.expand);
        if (F_status_is_error(main->setting.state.status)) return;
      }
    } // for

    main->setting.state.status = F_okay;
  }
#endif // _di_iki_read_process_wrap_prepend_

#ifdef __cplusplus
} // extern "C"
#endif
