/**
 * FLL - Level 3
 *
 * Project: IKI Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the process functionality.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_read_process_h
#define _iki_read_process_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Process a given buffer.
 *
 * This will print error messages, except for errors in callbacks.
 * The callbacks should handle their own error printing.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but nothing done.
 *
 *     F_interrupt (with error bit) on (exit) signal received.
 *
 *     Errors (with error bit) from: main.callback.process_buffer_ranges().
 *     Errors (with error bit) from: main.callback.process_buffer_ranges_whole().
 *     Errors (with error bit) from: main.callback.process_buffer_total().
 *
 * @see main.callback.process_buffer_ranges()
 * @see main.callback.process_buffer_ranges_whole()
 * @see main.callback.process_buffer_total()
 */
#ifndef _di_iki_read_process_buffer_
  extern void iki_read_process_buffer(iki_read_main_t * const main);
#endif // _di_iki_read_process_buffer_

/**
 * Append a new line at the end of the expand cache if the state status is F_okay.
 *
 * This is intended to be called immediately after iki_read_iki_process_buffer_at() and iki_read_eki_process_buffer_at() to handle the state status.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     Success from: f_string_dynamic_append()
 *
 *     Errors (with error bit) from: f_string_dynamic_append().
 *
 * @see f_string_dynamic_append()
 */
#ifndef _di_iki_read_process_buffer_at_newline_
  extern void iki_read_process_buffer_at_newline(iki_read_main_t * const main);
#endif // _di_iki_read_process_buffer_at_newline_

/**
 * Determine the range based on the --line parameter.
 *
 * If the --line parameter is not specified in the console arguments, then range is untouched.
 * The range.start will be greater than main->buffer.used if the --line is not found before buffer end is reached.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     F_true is returned if the range is processed.
 *     F_false is returned if the range is not processed.
 *     F_data_not if the range is processed, but the requested line is out of range.
 * @param range
 *   The range value to represent the --line values.
 */
#ifndef _di_iki_read_process_line_
  extern void iki_read_process_line(iki_read_main_t * const main, f_range_t * const range);
#endif // _di_iki_read_process_line_

/**
 * Process the append part of the wrap at the given index adding it to the expand cache, if valid.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but nothing added to the expand cache.
 *
 *     Errors (with error bit) from: f_string_dynamic_append_nulless()
 *
 *     F_failure (with error bit) for any other failure.
 * @param index
 *   The index within the setting->map_wrapss array to print.
 *
 * @see f_string_dynamic_append_nulless()
 */
#ifndef _di_iki_read_process_wrap_append_
  extern void iki_read_process_wrap_append(iki_read_main_t * const main, const f_number_unsigned_t index);
#endif // _di_iki_read_process_wrap_append_

/**
 * Print the prepend part of the wrap at the given index adding it to the expand cache, if valid.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This alters setting.state.status:
 *     F_okay on success.
 *     F_data_not on success, but nothing added to the expand cache.
 *
 *     Errors (with error bit) from: f_string_dynamic_append_nulless()
 *
 *     F_failure (with error bit) for any other failure.
 * @param index
 *   The index within the setting->map_wrapss array to print.
 *
 * @see f_string_dynamic_append_nulless()
 */
#ifndef _di_iki_read_process_wrap_prepend_
  extern void iki_read_process_wrap_prepend(iki_read_main_t * const main, const f_number_unsigned_t index);
#endif // _di_iki_read_process_wrap_prepend_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_read_process_h
